﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Telerik.Windows.Controls;
using Telerik.Windows.Controls.Diagrams;
using Telerik.Windows.Controls.Diagrams.Extensions;
using Telerik.Windows.DragDrop;

namespace MVVM
{
    public partial class MainPage : UserControl
    {
        private FileManager fileManager;

        static MainPage()
        {
            var saveBinding = new CommandBinding(DiagramCommands.Save, ExecuteSave, CanExecutedSave);
            var openBinding = new CommandBinding(DiagramCommands.Open, ExecuteOpen);

            CommandManager.RegisterClassCommandBinding(typeof(MainPage), saveBinding);
            CommandManager.RegisterClassCommandBinding(typeof(MainPage), openBinding);

        }

        public MainPage()
        {
            InitializeComponent();

            this.fileManager = new FileManager(this.xDiagram);
            this.DataContext = new CarsGraphSource();

            DragDropManager.AddDragInitializeHandler(xGallery, OnToolboxDragInitialize, true);
            DragDropManager.AddDropHandler(xDiagram, OnDiagramDrop, true);
        }

        private void OnContainerDrop(object sender, Telerik.Windows.DragDrop.DragEventArgs e)
        {
            Model draggedNode = (e.Data as DataObject).GetData(typeof(Model)) as Model;
            draggedNode.Position = this.xDiagram.TranslatePoint(e.GetPosition(this.xDiagram), this.xDiagram);
            Brand item = this.xDiagram.ContainerGenerator.ItemFromContainer(sender) as Brand;
            item.AddItem(draggedNode);
            e.Handled = true;
        }

        private void OnDiagramDrop(object sender, Telerik.Windows.DragDrop.DragEventArgs e)
        {
            Model draggedNode = (e.Data as DataObject).GetData(typeof(Model)) as Model;
            draggedNode.Position = this.xDiagram.TranslatePoint(e.GetPosition(this.xDiagram), this.xDiagram);
            (xDiagram.GraphSource as CarsGraphSource).AddNode(draggedNode);
            e.Handled = true;
        }


        private static void OnToolboxDragInitialize(object sender, DragInitializeEventArgs e)
        {
            RadDiagramToolboxItem toolboxItem = e.OriginalSource as RadDiagramToolboxItem;
            if (toolboxItem == null) return;
            Model node = new Model();
            if (toolboxItem.Content is GalleryItem && (toolboxItem.Content as GalleryItem).Shape != null)
            {
                node.Content = (toolboxItem.Content as GalleryItem).Header;
                node.Geometry = ((toolboxItem.Content as GalleryItem).Shape as RadDiagramShape).Geometry;
            }
            node.Height = toolboxItem.Height;
            node.Width = toolboxItem.Width;

            e.Data = node;
            e.Handled = true;
        }

        private static void CanExecutedSave(object sender, CanExecuteRoutedEventArgs e)
        {
            var owner = sender as MainPage;
            e.CanExecute = owner != null && owner.xDiagram != null && owner.xDiagram.Items.Count > 0;
        }

        private static void ExecuteSave(object sender, ExecutedRoutedEventArgs e)
        {
            var owner = sender as MainPage;
            if (owner != null)
                owner.fileManager.SaveToFile();
        }

        private static void ExecuteOpen(object sender, ExecutedRoutedEventArgs e)
        {
            var owner = sender as MainPage;
            if (owner != null)
            {
                (owner.DataContext as CarsGraphSource).InternalItems.Clear();
                (owner.DataContext as CarsGraphSource).InternalLinks.Clear();
                owner.fileManager.LoadFromFile();
            }
        }

        private void xDiagram_ItemsChanged(object sender, DiagramItemsChangedEventArgs e)
        {
            if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Add && e.NewItems.OfType<Brand>().Count() > 0)
            {
                RadDiagramContainerShape container = this.xDiagram.ContainerGenerator.ContainerFromItem(e.NewItems.OfType<Brand>().FirstOrDefault()) as RadDiagramContainerShape;
                if (container != null)
                    DragDropManager.AddDropHandler(container, OnContainerDrop, true);

            }
            if (e.Action == System.Collections.Specialized.NotifyCollectionChangedAction.Remove && e.OldItems.OfType<Brand>().Count() > 0)
            {
                RadDiagramContainerShape container = this.xDiagram.ContainerGenerator.ContainerFromItem(e.OldItems.OfType<Brand>().FirstOrDefault()) as RadDiagramContainerShape;
                if (container != null)
                    DragDropManager.RemoveDropHandler(container, OnContainerDrop);
            }
        }

    }
}
